<?php
declare(strict_types = 1);

/**
 * Gestion des appareils photos.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class Cameras
{
	/**
	 * Supprime des marques ou modèles d'appareils photos.
	 *
	 * @param string $type
	 *   Type accepté : 'brand' ou 'model'.
	 * @param mixed $cameras_id
	 *   Identifiant des marques.
	 *
	 * @return int
	 *   Retourne le nombre de lignes affectées,
	 *   ou -1 en cas d'erreur.
	 */
	public static function delete(string $type, $cameras_id): int
	{
		if (!is_array($cameras_id))
		{
			$cameras_id = [$cameras_id];
		}
		if (!$cameras_id)
		{
			return 0;
		}

		$cameras_id = DB::inInt($cameras_id);
		$sql = "DELETE FROM {cameras_" . $type . "s}
				 WHERE camera_" . $type . "_id IN ($cameras_id)";
		if (!DB::execute($sql))
		{
			return -1;
		}

		return DB::rowCount();
	}

	/**
	 * Édite les informations de plusieurs marques ou modèles d'appareils photos.
	 *
	 * @param string $type
	 *   Type accepté : 'brand' ou 'model'.
	 * @param array $update
	 *   Informations de mise à jour.
	 *
	 * @return int
	 *   Retourne le nombre de lignes affectées,
	 *   ou -1 en cas d'erreur.
	 */
	public static function edit(string $type, array $update): int
	{
		if (!in_array($type, ['brand', 'model']) || !$update)
		{
			return 0;
		}

		// Début de la transaction.
		if (!DB::beginTransaction())
		{
			goto fail;
		}

		// Récupération des informations utiles.
		$cameras_id = DB::inInt(array_keys($update));
		$sql = "SELECT *
				  FROM {cameras_" . $type . "s}
				 WHERE camera_" . $type . "_id IN ($cameras_id)";
		if (!DB::execute($sql))
		{
			goto fail;
		}
		$cameras = DB::fetchAll();

		// On détermine les informations à mettre à jour.
		$params = [];
		$n = 0;
		foreach ($cameras as &$i)
		{
			foreach (['name', 'url'] as $col)
			{
				if (isset($update[$i['camera_' . $type . '_id']][$col]))
				{
					$str = (string) $update[$i['camera_' . $type . '_id']][$col];
					$str = Utility::trimAll($str);
					$str = str_replace(',', '', $str);
					if ($col == 'url')
					{
						$str = App::getURLName($str);
					}
					if (!Utility::isEmpty($str) && $str !== $i['camera_' . $type . '_' . $col])
					{
						$params[$n][$col] = $str;
					}
				}
			}
			if (!empty($params[$n]))
			{
				foreach (['id', 'name', 'url'] as $col)
				{
					if (!isset($params[$n][$col]))
					{
						$params[$n][$col] = $i['camera_' . $type . '_' . $col];
					}
				}
				$n++;
			}
		}

		// Mise à jour de la base de données.
		if (!$params)
		{
			return 0;
		}
		$sql = "SELECT COUNT(*)
				  FROM {cameras_{$type}s}
				 WHERE camera_{$type}_id != :id
				   AND (camera_{$type}_name = :name OR camera_{$type}_url = :url)";
		$sql = "UPDATE {cameras_{$type}s}
				   SET camera_{$type}_name = :name,
					   camera_{$type}_url = :url
				 WHERE camera_{$type}_id = :id
				   AND ($sql) = 0";
		if (!DB::execute($sql, $params))
		{
			goto fail;
		}
		$updated = DB::rowCount();

		// Exécution de la transaction.
		if (!DB::commitTransaction())
		{
			goto fail;
		}

		return $updated;

		fail:
		DB::rollbackTransaction();
		return -1;
	}
}
?>